--[[
 MiniPet Addon
 Version 3.2
 $Revision: 77481 $

 Currently hosted by Breanni of Scarlet Crusade
 All credit for authoring this addon is listed below

 Visit WarcraftPets.com for all your minipet needs!
]]--

--[[
    MiniPet by Carra of Stormrage
    
    There are a number of events where the player looses his pet and we can notify him:
    -Entering the game                   >OK
    -Entering an instance                >OK
    -Dieing and getting back alive       >Usually OK :p
    -Using a Taxi Service                >OK
    -Teleporting/Summoning/Hearthstoning >OK but no summoning
    
    Based originally on MiniPetLeash (completely rewritten except for the events which are copied).
    
    Updated and converted to Ace2 by LordFarlander
]]--

local L = AceLibrary( "AceLocale-2.2" ):new( "MiniPet" );
local LBZ = AceLibrary( "LibBabble-Zone-3.0" ):GetLookupTable();

local BAG_TYPE_VANITYPET = 4096;

_G["BINDING_NAME_CLICK MiniPet_Button:LeftButton"] = L["Summon MiniPet"];
BINDING_HEADER_MINIPETHEADER = L["MiniPet"];

local options = {
    type = "group",
    args = {
        [L["toggleAutoHideButton"]] = {
            type = "toggle",
            name = L["Auto Hide Button"],
            desc = L["Hide the button when you click it?"],
            get = function()
                      return MiniPet.db.profile.Button.AutoHideButton;
                  end,
            set = function( val )
                      MiniPet.db.profile.Button.AutoHideButton = not MiniPet.db.profile.Button.AutoHideButton;
                  end,
        },
--        toggleDebug = {
--            type = "toggle",
--            name = "Debug",
--            desc = "Debug mode toggle",
--            get = function()
--                      return MiniPet.DEBUG;
--                  end,
--            set = function( val )
--                      MiniPet.DEBUG = not MiniPet.DEBUG;
--                  end,
--            },
        [L["toggleButton"]] = {
            type = "toggle",
            name = L["Show Button"],
            desc = L["Show/hide the button."],
            get = function()
                      return MiniPet.button:IsVisible()
                  end,
            set = function( val )
                      if( MiniPet.button:IsVisible() ) then
                          MiniPet:HideButton();
                      else
                          MiniPet:ShowButton();
                      end
                  end,
            },
        [L["toggleTextWarning"]] = {
            type = "toggle",
            name = L["Text Warning"],
            desc = L["Show text warnings to alert you to summon your pet."],
            get = function()
                      return MiniPet.db.profile.GiveTextWarning;
                  end,
            set = function( val )
                      MiniPet.db.profile.GiveTextWarning = not MiniPet.db.profile.GiveTextWarning;
                  end,
            },
        [L["toggleButtonWarning"]] = {
            type = "toggle",
            name = L["Button Warning"],
            desc = L["Flash the button to alert you to summon your pet."],
            get = function()
                      return MiniPet.db.profile.GiveButtonWarning;
                  end,
            set = function( val )
                      MiniPet.db.profile.GiveButtonWarning = not MiniPet.db.profile.GiveButtonWarning;
                  end,
            },
        [L["toggleMountWarning"]] = {
            type = "toggle",
            name = L["Mount Warning"],
            desc = L["Show an alert when you dismount from your flying mount."],
            get = function()
                      return MiniPet.db.profile.GiveMountWarning;
                  end,
            set = function( val )
                      MiniPet.db.profile.GiveMountWarning = not MiniPet.db.profile.GiveMountWarning;
                  end,
            },
        [L["toggleBG"]] = {
            type = "toggle",
            name = L["Battle Ground Check"],
            desc = L["Show minipet alerts while in a battleground."],
            get = function()
                      return MiniPet.db.profile.BattleGroundCheck;
                  end,
            set = function( val )
                      MiniPet.db.profile.BattleGroundCheck = not MiniPet.db.profile.BattleGroundCheck;
                  end,
            },
        [L["lockButton"]] = {
            type = "toggle",
            name = L["Lock Button Position"],
            desc = L["Lock/unlock the position of the button."],
            get = function()
                      return MiniPet.db.profile.Button.LockButton;
                  end,
            set = function( val )
                      MiniPet.db.profile.Button.LockButton = not MiniPet.db.profile.Button.LockButton;
                  end,
             },
        [L["suggestOnlySoulbound"]] = {
            type = "toggle",
            name = L["Suggest Only Soulbound MiniPets"],
            desc = L["Toggles whether only soulbound pets will be suggested for use."],
            get = function()
                      return MiniPet.db.profile.SuggestOnlySoulbound;
                  end,
            set = function( val )
                      MiniPet.db.profile.SuggestOnlySoulbound = not MiniPet.db.profile.SuggestOnlySoulbound;
                      MiniPet:GetPlayersPets();
                      MiniPet:ChooseRandomPet();
                  end,
            },
        [L["reagentCheck"]] = {
            type = "toggle",
            name = L["Check for reagented MiniPets that are missing their reagent"],
            desc = L["Toggles whether pets that require a reagent will be suggested when you have none of the required reagent."],
            get = function()
                      return MiniPet.db.profile.ReagentCheck;
                  end,
            set = function( val )
                      MiniPet.db.profile.ReagentCheck = not MiniPet.db.profile.ReagentCheck;
                      MiniPet:GetPlayersPets();
                      MiniPet:ChooseRandomPet();
                  end,
            },
        [L["equipmentCheck"]] = {
            type = "toggle",
            name = L["Check to make sure that equipment MiniPets are equipped"],
            desc = L["Toggles whether pets summoned by equipment will be suggested when the item is unequipped."],
            get = function()
                      return MiniPet.db.profile.EquipmentCheck;
                  end,
            set = function( val )
                      MiniPet.db.profile.EquipmentCheck = not MiniPet.db.profile.EquipmentCheck;
                      MiniPet:GetPlayersPets();
                      MiniPet:ChooseRandomPet();
                  end,
            },
        [L["forceQuestPet"]] = {
            type = "toggle",
            name = L["Force to pick a quest MiniPet if availiable"],
            desc = L["Toggles whether pets that are part of a quest are preferred instead of a random pet."],
            get = function()
                      return MiniPet.db.profile.ForceQuestPet;
                  end,
            set = function( val )
                      MiniPet.db.profile.ForceQuestPet = not MiniPet.db.profile.ForceQuestPet;
                      MiniPet:GetPlayersPets();
                      MiniPet:ChooseRandomPet();
                  end,
            },
        [L["list"]] = {
            type = "execute",
            name = L["List all MiniPets"],
            desc = L["Show a list with all of your minipets."],
            func = "ListPets",
            },
        [L["relist"]] = {
            type = "execute",
            name = L["Refresh MiniPet list"],
            desc = L["Force a refresh of the list of all your pets."],
            func = "GetPlayersPets",
            },
        },
    };

MiniPet = AceLibrary( "AceAddon-2.0" ):new( "AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0" );
MiniPet.version = tonumber( string.match( "$Rev: 77481 $", "(%d+)" ) );
MiniPet:RegisterChatCommand( "/MiniPet", "/mp", options );
MiniPet:RegisterDB( "MiniPetDB", "MiniPetDBPC" );
MiniPet:RegisterDefaults( "profile", {
        BattleGroundCheck = true,
        GiveTextWarning = true,
        GiveButtonWarning = true,
        GiveMountWarning = true,
        SuggestOnlySoulbound = true,
        ReagentCheck = true,
        EquipmentCheck = true,
        ForceQuestPet = true,
        Button = {
            AutoHideButton = true,
            LockButton = false,
            ButtonGroup = {
                SkinID = "Blizzard",
                Gloss = nil,
                Backdrop = nil,
                Colors = {},
            },
            posX = 20,
            posY = 20,
        },
    } );

local PetAndMountDatabase = LibStub( "PetAndMountDatabase-1.0", true );
if( not PetAndMountDatabase ) then
    error( "MiniPet requires PetAndMountDatabase-1.0" );
    return;
end--if

local LibHelper = LibStub( "LibLordFarlander-1.0", true );
if( not LibHelper ) then
    error( "MiniPet requires LibLordFarlander-1.0" );
    return;
end--if

local ItemList = LibStub( "LibLordFarlander-ItemList-1.0", true );
if( not ItemList ) then
    error( "MiniPet requires LibLordFarlander-ItemList-1.0" );
    return;
end--if

local LibKeyBound = LibStub( "LibKeyBound-1.0", true );
if( not LibKeyBound ) then
    error( "MiniPet requires LibKeyBound-1.0" );
    return;
end--if

local LibStickyFrames = LibStub( "LibStickyFrames-2.0", true );
if( not LibStickyFrames ) then
    error( "Mount requires LibStickyFrames-2.0" );
    return;
end--if

local LibButtonFacade = LibStub( "LibButtonFacade", true );
        
--Global Values
MiniPet.DEBUG = false;            --true = Debugging Mode
MiniPet.dprint = LibHelper.dprint;
MiniPet.button = nil;

MiniPet.UpdateInterval      = 0.2;   --Time between two flashes of the button in seconds
MiniPet.TotalFlashTime      = 10;    --Total time to flash the button in seconds

--DO NOT CHANGE THESE VALUES
MiniPet.PlayersPets         = {};    --List of the players pet
MiniPet.CurrentPet          = nil;   --Pet on the Button
MiniPet.BoardedTaxi         = false;
MiniPet.StartedHearthstone  = false;
MiniPet.HearthstoneComplete = false;
MiniPet.PlayerDied          = false;
MiniPet.PlayerEnteringWorld = true;
MiniPet.ReadyToInitialize   = false;
MiniPet.PetListOkay         = false;
MiniPet.LastPet             = nil;
MiniPet.QuestPet            = nil;

MiniPet.ShowButtonImg       = false; --Goes on and off if button flashes
MiniPet.FlashTime           = 0;     --Time since button flashes
MiniPet.FlashButton         = false; --On when the button flashes

MiniPet.Dirty               = false;

MiniPet.keyBoundMode = false;

local MINIPET_BATTLEGROUNDS = table.concat( {
    	LBZ["Warsong Gulch"],
    	LBZ["Arathi Basin"],
    	LBZ["Alterac Valley"],
    	LBZ["Eye of the Storm"],
	}, "|" );

function MiniPet:OnEnable()
    --Register the Events
    self:RegisterEvent( "BAG_UPDATE",
        function( bagID )
            MiniPet.Dirty = true;
            if( not MiniPet.button:IsVisible() ) then
                MiniPet:ScheduleEvent( MiniPet.OnUpdate, 0 );
            end--if
        end );
    self:RegisterEvent( "PLAYER_ENTERING_WORLD",
        function()
            MiniPet.PlayerDied = UnitIsDeadOrGhost( "player" );
            MiniPet.BoardedTaxi = UnitOnTaxi( "player" );
            MiniPet.PlayerEnteringWorld = false;
            MiniPet:WarnPlayer();
        end );
    self:RegisterEvent( "TAXIMAP_CLOSED",
        function()
            MiniPet.BoardedTaxi = true;
        end );
    self:RegisterEvent( "PLAYER_CONTROL_GAINED",
        function()
            if( MiniPet.BoardedTaxi ) then
                MiniPet.BoardedTaxi = false;
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "PLAYER_DEAD",
        function()
            MiniPet:dprint( "MiniPet: PlayerDied" );
            MiniPet.PlayerDied = true;
        end );
    self:RegisterEvent( "PLAYER_ALIVE",
        function()
            MiniPet.BoardedTaxi = UnitOnTaxi( "player" );
            if( MiniPet.HearthstoneComplete ) then
                MiniPet.HearthstoneComplete = false;
                MiniPet:dprint( "MiniPet: Activating Pet After Hearthstone!" );
                MiniPet:WarnPlayer();
            end--if
            if( MiniPet.PlayerDied ) then
                MiniPet.PlayerDied = false;
                MiniPet:dprint( "MiniPet: no longer dead!" );
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "PLAYER_UNGHOST",
        function()
            if( MiniPet.PlayerDied ) then
                MiniPet.PlayerDied = false;
                MiniPet:dprint( "MiniPet: no longer ghost!" );
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "UNIT_SPELLCAST_INTERRUPTED",
        function( unit, _, _ )
            if( unit == "player" ) then
                if( MiniPet.StartedHearthstone ) then
                    MiniPet:dprint( "MiniPet: Interrupted Teleporting!" );
                    MiniPet.StartedHearthstone = false;
                end--if
            end--if
        end );
    self:RegisterEvent( "UNIT_SPELLCAST_SUCCEEDED",
        function( unit, spellName, _ )
            if( unit == "player" ) then
                if( MiniPet.StartedHearthstone ) then
                    if( MiniPet.IsATeleportSpell( spellName ) ) then
                        MiniPet:dprint( "MiniPet: Finished Teleporting!" );
                        MiniPet.StartedHearthstone = false;
                        MiniPet.HearthstoneComplete = true;
                        MiniPet:WarnPlayer();
                    end--if
                end--if
            end--if
        end );
    self:RegisterEvent( "UNIT_SPELLCAST_SENT",
        function( unit, spellName, _, _ )
            if( unit == "player" ) then
                MiniPet:dprint( "Spellcast: " .. spellName );
                MiniPet.StartedHearthstone = false;
                if( MiniPet.IsATeleportSpell( spellName ) ) then
                    MiniPet:dprint( "MiniPet: Started Teleporting!" );
                    MiniPet.StartedHearthstone = true;
                end--if
            end--if
        end );
    self:RegisterEvent( "SpecialEvents_Dismounted",
        function( mountName, speed, flying )
            if( flying and MiniPet.db.profile.GiveMountWarning ) then
                MiniPet:WarnPlayer();
            end--if
        end );
    self:RegisterEvent( "PLAYER_REGEN_ENABLED",
        function()
            MiniPet:SetButton( MiniPet.CurrentPet );
        end );
        
    -- LibKeyBound support
    LibKeyBound.RegisterCallback( self, "LIBKEYBOUND_ENABLED" );
    LibKeyBound.RegisterCallback( self, "LIBKEYBOUND_DISABLED" );
    LibKeyBound.RegisterCallback( self, "LIBKEYBOUND_MODE_COLOR_CHANGED" );

    self.button = MiniPet_Button;
    LibStickyFrames:RegisterFrame( self.button );
    LibStickyFrames:SetFrameGroup( self.button, true );
    self.button.tooltip = MiniPet_Tooltip;

    self.button.tooltip.SetFromListEntry = ItemList.SetTooltipFromListEntry;
    self.button.SetFromListEntry = ItemList.SetButtonFromListEntry;
    self.button.SetImage = LibHelper.SetButtonImage;
    self.button.cooldown = _G[self.button:GetName() .. "Cooldown"];
    self.button.GetHotkey = function() return GetBindingKey( "BINDING_NAME_CLICK MiniPet_Button:LeftButton" ) end;
    self.button.GetActionName = function() return L["MiniPet"] end;
    
    -- LibStickyFrames support
    LibStickyFrames.RegisterCallback( self, "OnStickToFrame", MiniPet.OnStickToFrame );
    LibStickyFrames.RegisterCallback( self, "OnStopFrameMoving", MiniPet.OnStickToFrame );
    if( self.db.profile.Button.StickToFrame and _G[self.db.profile.Button.StickToFrame] ) then
        LibStickyFrames:SetFramePoints( button, self.db.profile.Button.StickPoint, _G[self.db.profile.Button.StickToFrame], self.db.profile.Button.StickToPoint, self.db.profile.Button.StickToX, self.db.profile.Button.StickToY );
    end--if

    if( LibButtonFacade ) then
        self.button.icon = _G[self.button:GetName() .. "Icon"];
        LibButtonFacade:Group( "MiniPet" ):AddButton( self.button );
        LibButtonFacade:Group( "MiniPet" ):Skin( self.db.profile.Button.ButtonGroup.SkinID, self.db.profil.Button.ButtonGroup.Gloss, self.db.profile.Button.ButtonGroup.Backdrop, self.db.profile.Button.ButtonGroup.Colors );
        LibButtonFacade:RegisterSkinCallback( "MiniPet", self.SkinChanged, self );
    end--if

    --Upgrade check
    if( self.db.profile.LastVersion == nil ) then
        --Upgrading from 2.0 or earlier, import the old settings
        self.db.profile.BattleGroundCheck = MiniPet_BattleGroundCheck;
        self.db.profile.GiveTextWarning = MiniPet_GiveTextWarning;
        self.db.profile.GiveButtonWarning = MiniPet_GiveButtonWarning;
        self.db.profile.Button.AutoHideButton = MiniPet_HideButton;
    end--if
    if( self.db.profile.AutoHideButton ~= nil ) then
        self.db.profile.Button.AutoHideButton = self.db.profile.AutoHideButton;
        self.db.profile.AutoHideButton = nil;
    end--if
    if( self.db.profile.LockButton ~= nil ) then
        self.db.profile.Button.LockButton = self.db.profile.LockButton;
        self.db.profile.LockButton = nil;
    end--if
    if( self.db.profile.ButtonGroup ~= nil ) then
        self.db.profile.Button.ButtonGroup = self.db.profile.ButtonGroup;
        self.db.profile.ButtonGroup = nil;
    end--if
    if( self.db.profile.FadeOptions ~= nil ) then
        self.db.profile.FadeOptions = nil;
    end--if
    if( self.db.profile.LastVersion ~= self.version ) then
        self.db.profile.LastVersion = self.version;
    end--if

    self.Dirty = true;
    --Create the Pet Inventory
    self:GetPlayersPets();
    --Set a Pet
    self:ChooseRandomPet();
end--MiniPet:OnEnable();

--For ButtonFacade support
function MiniPet:SkinChanged( SkinID, Gloss, Backdrop, barKey, buttonKey, Colors )
    self.db.profile.Button.ButtonGroup.SkinID = SkinID;
    self.db.profile.Button.ButtonGroup.Gloss = Gloss;
    self.db.profile.Button.ButtonGroup.Backdrop = Backdrop;
    self.db.profile.Button.ButtonGroup.Colors = Colors;
end--MiniPet:SkinChanged( SkinID, Gloss, Backdrop, barKey, buttonKey, Colors )

--For LibKeyBound support
function MiniPet:ColorButton()
    local overlayFrame = self.button;

    if( self.keyBoundMode ) then
        overlayFrame:SetBackdropColor( LibKeyBound:GetColorKeyBoundMode() );
    end--if
end--MiniPet:ColorButton()

--For LibKeyBound support
function MiniPet:LIBKEYBOUND_ENABLED()
	self.keyBoundMode = true;
	self:ColorButton();
end--MiniPet:LIBKEYBOUND_ENABLED()

--For LibKeyBound support
function MiniPet:LIBKEYBOUND_DISABLED()
	self.keyBoundMode = false;
	self:ColorButton();
end--MiniPet:LIBKEYBOUND_DISABLED()

--For LibKeyBound support
function MiniPet:LIBKEYBOUND_MODE_COLOR_CHANGED()
	self:ColorButton()
end--MiniPet:LIBKEYBOUND_MODE_COLOR_CHANGED()

--For LibStickyFrames support
function MiniPet:StartFrameMoving()
    LibStickyFrames:StartFrameMoving( self.button );
end--MiniPet:StartFrameMoving()

--For LibStickyFrames support
function MiniPet:StopFrameMoving()
    LibStickyFrames:StopFrameMoving( self.button );
end--MiniPet:StartFrameMoving()

--For LibStickyFrames support
function MiniPet:HideButton()
    self.button:Hide();
    LibStickyFrames:SetFrameHidden( self.button, true );
end--MiniPet:HideButton()

--For LibStickyFrames support
function MiniPet:ShowButton()
    self.button:Show();
    LibStickyFrames:SetFrameHidden( self.button, false );
end--MiniPet:ShowButton()

--For LibSticyFrames support
function MiniPet.OnStickToFrame( event, frame, point, stickToFrame, stickToPoint, stickToX, stickToY )
    local x, y = frame:GetLeft(), frame:GetBottom()
    local s = frame:GetEffectiveScale()
    local Button = MiniPet.db.profile.Button;

    LibStickyFrames:SetFramePoints( frame, point, stickToFrame, stickToPoint, stickToX, stickToY );
    Button.posX = x * s;
    Button.posY = y * s;
    Button.StickPoint = point;
    Button.StickToFrame = stickToFrame and stickToFrame:GetName() or nil;
    Button.StickToPoint = stickToPoint;
    Button.StickToX = stickToX;
    Button.StickToY = stickToY;
end--MiniPet.OnStickToFrame( event, frame, point, stickToFrame, stickToPoint, stickToX, stickToY )

function MiniPet:OnUpdate( elapsed )
    if( self == nil ) then
        self = MiniPet;
    end--if
    if( ( UnitName( "player" ) ~= UNKNOWNOBJECT ) and self.ReadyToInitialize ) then
        self:dprint( "MiniPet: PlayerEnteringWorld and ALIVE" );
        self.PlayerEnteringWorld = false;
        self.ReadyToInitialize = false;
        self:WarnPlayer();
    end--f
    
    if( self.Dirty ) then
        self.Dirty = false;
        if( self:HasMiniPetListChanged() ) then
            self:ChooseRandomPet();
        end--if
        self:SetButton( self.CurrentPet );
    end--if

    --Code to let the button flash
    --elapsed = amount of seconds that passed since last update.
    --every updateinterval, change the button image
    --until you reach totalflashtime
    if( self.FlashButton ) then
        if( self.button.TimeSinceLastUpdate == nil ) then
            self.button.TimeSinceLastUpdate = 0;
            self.FlashTime = 0;
        end--if
        self.button.TimeSinceLastUpdate = self.button.TimeSinceLastUpdate + elapsed;
        self.FlashTime = self.FlashTime + elapsed;
        if( self.FlashTime < self.TotalFlashTime ) then
            if( self.button.TimeSinceLastUpdate > self.UpdateInterval ) then
                if( self.ShowButtonImg and ( self.CurrentPet ~= nil ) ) then
                    self.button:SetFromListEntry( self.CurrentPet );
                    self.ShowButtonImg = false;
                else
                    self.button:SetImage( "Interface\\AddOns\\MiniPet\\Img\\MiniPet" );
                    self.ShowButtonImg = true;
                end--if
                self.button.TimeSinceLastUpdate = 0;
            end--if
        else
            self.FlashButton = false;
            self:SetButton( self.CurrentPet );
            self.FlashTime = 0;
        end--if
    end--if
end--MiniPet:OnUpdate( elapsed )

----------------------------
--+Pet/Inventory Functions--
--Pick a random pet and set the button.
function MiniPet:ChooseRandomPet()
    if( self.db.profile.ForceQuestPet and self.QuestPet ) then
        self.CurrentPet = self.QuestPet;
    else
        self.CurrentPet = ItemList.GetRandomItemFromList( self.PlayersPets, self.CurrentPet );
    end--if
    self:SetButton( self.CurrentPet );
end--MiniPet:ChooseRandomPet()

--Gets all your pets from your inventory.
function MiniPet:GetPlayersPets()
    self.PlayersPets = {};
    self.QuestPet = nil;

    local playerPetList = self.PlayersPets;
    local itemLink;
    local reagented;
    local equipmentSlot;
    local opSuggestOnlySoulbound = self.db.profile.SuggestOnlySoulbound;
    local opReagentCheck = self.db.profile.ReagentCheck;
    local opEquipmentCheck = self.db.profile.EquipmentCheck;

    for bag = 0, NUM_BAG_SLOTS do
        for slot = 1, GetContainerNumSlots( bag ) do
            itemLink = GetContainerItemLink( bag, slot );
            
            if( itemLink ) then
                if( PetAndMountDatabase:ItemInSet( itemLink, "MiniPets" ) ) then
                    if( opReagentCheck ) then
                        reagented = PetAndMountDatabase:ItemInSet( itemLink, "MiniPets.Reagented" );
                    else
                        reagented = nil;
                    end--if
                    if( ( ( not opSuggestOnlySoulbound ) or ( LibHelper.ItemIsSoulbound( bag, slot ) ) ) and
                        ( ( not opReagentCheck ) or ( not reagented ) or ( GetItemCount( reagented ) > 0 ) ) and
                        ( ( not opEquipmentCheck ) or ( not PetAndMountDatabase:ItemInSet( itemLink, "MiniPets.Equipment" ) ) ) ) then
                        ItemList.AddBagItemToList( playerPetList, bag, slot );
                        if( self.db.profile.ForceQuestPet ) then
                            local questID = PetAndMountDatabase:ItemInSet( itemLink, "MiniPets.Quest" );
    
                            if( ( questID and self:HasQuestAndNotComplete( questID ) ) or
                                ( ( self.QuestPet == nil ) and PetAndMountDatabase:ItemInSet( itemLink, "MiniPets.Children" ) ) ) then
                                self.QuestPet = ItemList.BuildBagItem( bag, slot );
                            end--if
                        end--if
                    end--if
                elseif( GetItemFamily( itemLink ) and ( bit.band( GetItemFamily( itemLink ), BAG_TYPE_VANITYPET ) ~= 0 ) ) then
                    -- Assume is an unknown Normal Minipet
                    ItemList.AddBagItemToList( playerPetList, bag, slot );
                end--if
            end--if
        end--for
    end--for
    
    local iterator, handle = PetAndMountDatabase:IterateSet( "MiniPets.Equipment" );
    local pet, equipmentSlot = iterator( handle );

    while pet do
        if( opEquipmentCheck and ( LibHelper.GetItemIDFromLink( GetInventoryItemLink( "player", GetInventorySlotInfo( equipmentSlot ) ) ) == pet ) ) then
            ItemList.AddInventoryItemToList( playerPetList, equipmentSlot );
        end--if
        pet, equipmentSlot = iterator( handle );
    end--for
end--MiniPet:GetPlayerPets()

--Prints out a list with all your pets.
function MiniPet:ListPets()
    self:Print( L["MiniPet, List of your pets:"] );
    for _, pet in pairs( self.PlayersPets ) do
        self:Print( "-|cFF00FFFF" .. pet:GetName() );
    end--for
end--MiniPet:listPets()
---Pet/Invenroty Functions--
----------------------------

-------------------
--+Help Functions--
function MiniPet.IsATeleportSpell( spellName )
    local iterator, handle = PetAndMountDatabase:IterateSet( "Spells.Teleport" );
    local spell, _ = iterator( handle );

    while spell do
        if( spell < 0 ) then
            spell = -1 * spell;
        end--if
        if( spellName == GetSpellInfo( spell ) ) then
            return true;
        end--if
        spell, _ = iterator( handle );
    end--for
    return false;
end--MiniPet.IsATeleportSpell( spellName )

function MiniPet:HasMiniPetListChanged()
    local prePetCount = #self.PlayersPets;
    local prePetList =  {};

    LibHelper.CopyTable( prePetList, self.PlayersPets );

    self:GetPlayersPets();

    -- differences in sizes always means the list has changed
    if( ( prePetCount ~= #self.PlayersPets ) or ( not self.PetListOkay ) ) then
        self.PetListOkay = true;
        self:dprint( "Number of minipets changed" );
        return true;
    end--if
    self.PetListOkay = true;
    return not LibHelper.CompareTables( prePetList, self.PlayersPets );
end--MiniPet:HasMiniPetListChanged()

--Check if a player is in a battleground
function MiniPet.IsInBattleground()
    return MINIPET_BATTLEGROUNDS:find( GetZoneText() ) ~= nil;
end--MiniPet.IsInBattleground()

function MiniPet.HasQuestAndNotComplete( questID )
	local numEntries, _ = GetNumQuestLogEntries();

	if( numEntries > 0 ) then
		for id = 1, numEntries do
            local _, _, _, _, _, _, complete, _ = GetQuestLogTitle( id );
            
            if( ( not complete ) and ( LibHelper.GetItemIDFromLink( GetQuestLink( id ) ) == questID ) ) then
                return true;
            end--if
        end--for
    end--if
    return false;
end--Minipet.GetQuestID( questName )
---Help Functions--
-------------------

----------------------
--+Message Functions--
--Give the player a reminder.
function MiniPet:WarnPlayer()
    local charDB = self.db.char;

    if( ( not ( self.IsInBattleground() and charDB.BattleGroundCheck ) ) and ( not self.PlayerDied ) and ( not self.BoardedTaxi ) ) then--BG + option check
        self.FlashButton = charDB.GiveButtonWarning;
        if( charDB.GiveTextWarning ) then
            if( #self.PlayersPets > 0 ) then    --No use if player isn't cool and gots no pets!
                self.wprint( L["ALERT: Time to summon your minipet!"] );
            end--if
        end--if
        if( charDB.Button.AutoHideButton and ( not self.button:IsVisible() ) ) then
            self:ShowButton();
        end--if
    end--if
end--MiniPet:WarnPlayer()

--Shows a warning in the middle of the screen.
function MiniPet.wprint( msg )
    MinipetTextFrameText:SetTextColor( 1.0, 0.1, 0.1 );
    MinipetTextFrameText:SetText( msg );
    MinipetTextFrame.startTime = GetTime();
    MinipetTextFrame:Show();
end--MiniPet:wprint( msg )
---Message Functions
----------------------

-----------------
--+UI Functions--
function MiniPet:SetGameTooltips()
    local mpButton = self.button;

    if( not self.keyBoundMode ) then
        local pet = self.CurrentPet;
        local mpTooltip = mpButton.tooltip;
    
        mpTooltip:SetOwner( mpButton, "RIGHT" );
        mpTooltip:SetBackdropColor( 0.0, 0.0, 0.0, 1.0 );
        if( pet == nil ) then
            mpTooltip:SetText( L["Get a minipet for some company!"] );
        else
            mpTooltip:SetFromListEntry( pet );
        end--if
    end--if
    -- LibKeyBound support
    if( mpButton.GetHotkey ) then
       LibKeyBound:Set( mpButton );
    end--if
end--MiniPet:SetGameTooltips()

function MiniPet:SetButton( pet )
    local mpButton = self.button;

    if( not UnitAffectingCombat( "player" ) ) then
        if( pet == nil ) then
            mpButton.cooldown:Hide();
            mpButton:SetAttribute( "type1", "item" );
            mpButton:SetAttribute( "*bag1", nil );
            mpButton:SetAttribute( "*slot1", nil );
            mpButton:SetAttribute( "*item1", nil );

            mpButton:SetImage( "Interface\\AddOns\\MiniPet\\Img\\MiniPet" );

            self:dprint( "No pets" );
        else
            mpButton:SetFromListEntry( pet, not LibHelper.CompareTables( self.LastPet, pet ) );
        end--if
        self.LastPet = pet;
    end--if
end--MiniPet:SetButton( pet )
---UI Functions--
-----------------
